# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) 2020
#
# See the LICENSE file for details
# see the AUTHORS file for authors
# ----------------------------------------------------------------------

#--------------------
# System wide imports
# -------------------

import os.path
import logging

# ---------------------
# Third party libraries
# ---------------------


#--------------
# local imports
# -------------

from .        import AZOTEA_CFG_DIR
from .config  import load_config_file
from .image   import REGISTERED, STATS_COMPUTED, METADATA_UPDATED, DARK_SUBSTRACTED
from .image   import NO_CHANGES, CAMERA_CHANGES, OBSERVER_CHANGES

# ----------------
# Module constants
# ----------------

# values for the 'state' column in table


# -----------------------
# Module global variables
# -----------------------

log = logging.getLogger("azotea")

# -----------------------
# Module global functions
# -----------------------


def do_change(connection, key, state, change):
	config_path  = os.path.join(AZOTEA_CFG_DIR, key + '.ini')
	file_options = load_config_file(config_path)
	observer     = file_options['observer']
	row          = {'observer': observer, 'state': state }
	cursor       = connection.cursor()
	cursor.execute('''
			SELECT meta_changes
			FROM image_t
			WHERE observer = :observer
			AND state > :state
			LIMIT 1
			''', row)
	flags = cursor.fetchone()
	if flags is not None:
		flags = flags[0]
		flags += change
		row['changes'] = flags
		cursor.execute(
			'''
			UPDATE image_t
			SET state = :state, meta_changes = :changes
			WHERE observer == :observer
			AND state > :state
			''', row)
		connection.commit()
	if cursor.rowcount > 0:
		log.info("Updated processing state of %03d images for %s", cursor.rowcount, key)
	

# =====================
# Command esntry points
# =====================


def changed_observer(connection, options):
	log.info("Changed observer metadata in %s", options.key)
	do_change(connection, options.key, STATS_COMPUTED, OBSERVER_CHANGES)

def changed_location(connection, options):
	log.info("Changed location metadata in %s", options.key)
	do_change(connection, options.key, STATS_COMPUTED, OBSERVER_CHANGES)

def changed_camera(connection, options):
	log.info("Changed camera metadata in %s", options.key)
	do_change(connection, options.key, STATS_COMPUTED, CAMERA_CHANGES)

def changed_image(connection, options):
	log.info("Changed image metadata in %s", options.key)
	do_change(connection, options.key, REGISTERED, NO_CHANGES)



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) 2020
#
# See the LICENSE file for details
# see the AUTHORS file for authors
# ----------------------------------------------------------------------


# ----------
# Exceptions
# ----------

class NoUserInfoError(Exception):
    '''Working Directory does not contain user configuration data.'''
    def __str__(self):
        s = self.__doc__
        if self.args:
            s = "{0} \n".format(s, self.args[0])
        s = '{0}.'.format(s)
        return s

class MixingCandidates(Exception):
    '''Images processed in different directories.'''
    def __str__(self):
        s = self.__doc__
        if self.args:
            s = "{0} \n".format(s, self.args[0])
        s = '{0}.'.format(s)
        return s

class ConfigError(ValueError):
    '''This camera model is not yet supported by AZOTEA'''
    def __str__(self):
        s = self.__doc__
        if self.args:
            s = "{0}: '{1}'".format(s, self.args[0])
        s = '{0}.'.format(s)
        return s

class MetadataError(ValueError):
    '''Error reading EXIF metadata for image'''
    def __str__(self):
        s = self.__doc__
        if self.args:
            s = "{0}: '{1}'".format(s, self.args[0])
        s = '{0}.'.format(s)
        return s

class TimestampError(ValueError):
    '''EXIF timestamp not supported by AZOTEA'''
    def __str__(self):
        s = self.__doc__
        if self.args:
            s = "{0}: '{1}'".format(s, self.args[0])
        s = '{0}.'.format(s)
        return s



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) 2020
#
# See the LICENSE file for details
# see the AUTHORS file for authors
# ----------------------------------------------------------------------

#--------------------
# System wide imports
# -------------------

import sys
import argparse
import sqlite3
import os
import os.path
import logging
import logging.handlers
import traceback
import shutil

#--------------
# local imports
# -------------

from .           import *
from .exceptions import *
from .           import __version__
from .config     import load_config_file, merge_options 
from .utils      import chop, Point, ROI, open_database, create_database
from .cfgcmds    import config_global, config_camera
from .database   import database_clear, database_purge, database_backup
from .backup     import backup_list, backup_delete, backup_restore
from .image      import image_list, image_export, image_reduce
from .reorg      import reorganize_images
from .session    import session_current, session_list
from .changed    import changed_observer, changed_location, changed_camera, changed_image

# -----------------------
# Module global variables
# -----------------------

log = logging.getLogger("azotea")

# -----------------------
# Module global functions
# -----------------------

def configureLogging(options):
	if options.verbose:
		level = logging.DEBUG
	elif options.quiet:
		level = logging.WARN
	else:
		level = logging.INFO
	
	log.setLevel(level)
	# Log formatter
	#fmt = logging.Formatter('%(asctime)s - %(name)s [%(levelname)s] %(message)s')
	fmt = logging.Formatter('%(asctime)s [%(levelname)s] %(message)s')
	# create console handler and set level to debug
	if not options.no_console:
		ch = logging.StreamHandler()
		ch.setFormatter(fmt)
		ch.setLevel(level)
		log.addHandler(ch)
	# Create a file handler Suitable for logrotate usage
	if options.log_file:
		#fh = logging.handlers.WatchedFileHandler(options.log_file)
		fh = logging.handlers.TimedRotatingFileHandler(options.log_file, when='midnight', interval=1, backupCount=365)
		fh.setFormatter(fmt)
		fh.setLevel(level)
		log.addHandler(fh)


def python2_warning():
	if sys.version_info[0] < 3:
		log.warning("This software des not run under Python 2 !")


def setup(options):
	configureLogging(options)
	python2_warning()
	if not os.path.exists(AZOTEA_BASE_DIR):
		log.info("Creating {0} directory".format(AZOTEA_BASE_DIR))
		os.mkdir(AZOTEA_BASE_DIR)
	if not os.path.exists(AZOTEA_CFG_DIR):
		log.info("Creating {0} directory".format(AZOTEA_CFG_DIR))
		os.mkdir(AZOTEA_CFG_DIR)
	if not os.path.exists(AZOTEA_CSV_DIR):
		log.info("Creating {0} directory".format(AZOTEA_CSV_DIR))
		os.mkdir(AZOTEA_CSV_DIR)
	if not os.path.exists(AZOTEA_DB_DIR):
		log.info("Creating {0} directory".format(AZOTEA_DB_DIR))
		os.mkdir(AZOTEA_DB_DIR)
	if not os.path.exists(AZOTEA_BAK_DIR):
		log.info("Creating {0} directory".format(AZOTEA_BAK_DIR))
		os.mkdir(AZOTEA_BAK_DIR)
	if not os.path.exists(AZOTEA_LOG_DIR):
		log.info("Creating {0} directory".format(AZOTEA_LOG_DIR))
		os.mkdir(AZOTEA_LOG_DIR)
	if not os.path.exists(DEF_CONFIG):
		shutil.copy2(DEF_CONFIG_TPL, DEF_CONFIG)
		log.info("Created {0} file, please review it".format(DEF_CONFIG))



def mkrect1(text):
	'''Make a rectangle of width and height'''
	l = chop(text,',')
	return ROI( x1=0, x2=int(l[0]), y1=0, y2=int(l[1]))


# =================== #
# THE ARGUMENT PARSER #
# =================== #

def createParser():
	# create the top-level parser
	name = os.path.split(os.path.dirname(sys.argv[0]))[-1]
	parser    = argparse.ArgumentParser(prog=name, description="AZOTEA analysis tool")

	# Global options
	parser.add_argument('--version', action='version', version='{0} {1}'.format(name, __version__))
	group1 = parser.add_mutually_exclusive_group()
	group1.add_argument('-v', '--verbose', action='store_true', help='Verbose output.')
	group1.add_argument('-q', '--quiet',   action='store_true', help='Quiet output.')
	parser.add_argument('-nk','--no-console', action='store_true', help='Do not log to console.')
	parser.add_argument('--log-file', type=str, default=None, help='Optional log file')
	parser.add_argument('--camera', type=str, default=DEF_CAMERA, help='Optional alternate camera configuration file')
	parser.add_argument('--config', type=str, default=DEF_CONFIG, help='Optional alternate global configuration file')

	# --------------------------
	# Create first level parsers
	# --------------------------

	subparser = parser.add_subparsers(dest='command')

	parser_init     = subparser.add_parser('init', help='init command')
	parser_config   = subparser.add_parser('config', help='config commands')
	parser_image    = subparser.add_parser('image', help='image commands')
	parser_database = subparser.add_parser('database', help='database commands (mostly mainteinance)')
	parser_back     = subparser.add_parser('backup', help='backup management')
	parser_reorg    = subparser.add_parser('reorganize', help='reorganize commands')
	parser_session  = subparser.add_parser('session', help='session commands')
	parser_changed  = subparser.add_parser('changed', help='notify change commands (only for automatized use)')
   
	# ------------------------------------------
	# 'init' does not have a second level parser
	# ------------------------------------------

	# -----------------------------------------
	# Create second level parsers for 'changed'
	# -----------------------------------------

	subparser = parser_changed.add_subparsers(dest='subcommand')

	cho = subparser.add_parser('observer', help="Notify changes of observer metadata")
	cho.add_argument('-k', '--key',  type=str, required=True, help='who changed')
	
	chl = subparser.add_parser('location', help="Notify changes of location metadata")
	chl.add_argument('-k', '--key',  type=str, required=True, help='who changed')

	chc = subparser.add_parser('camera',   help="Notify changes of camera metadata")
	chc.add_argument('-k', '--key',  type=str, required=True, help='who changed')

	chi = subparser.add_parser('image',    help="Notify changes of image metadata")
	chi.add_argument('-k', '--key',  type=str, required=True, help='who changed')
	

	# -----------------------------------------
	# Create second level parsers for 'database'
	# -----------------------------------------

	subparser = parser_database.add_subparsers(dest='subcommand')

	dbc = subparser.add_parser('clear',  help="Clears the database (MAINTENANCE ONLY!)")
	dbcex = dbc.add_mutually_exclusive_group(required=True)
	dbcex.add_argument('-l', '--last',  action='store_true', help='clear last session')
	dbcex.add_argument('-a', '--all',   action='store_true' , help='clear all data')
	
	
	dbp = subparser.add_parser('purge',  help="Purge the database  (MAINTENANCE ONLY!)")

	dbp = subparser.add_parser('backup',  help="Database backup")
	dbp.add_argument('--comment', type=str,  help='Optional comment')

	# ----------------------------------------
	# Create second level parsers for 'backup'
	# ----------------------------------------

	subparser = parser_back.add_subparsers(dest='subcommand')

	bkl = subparser.add_parser('list',  help="List database backups")
	
	bkd = subparser.add_parser('delete',  help="Delete a given backup")
	bkd.add_argument('--bak-file', type=str, required=True , help='Backup file to deleta')

	bkr = subparser.add_parser('restore',  help="Restore database from backup")
	bkr.add_argument('--bak-file', type=str, required=True , help='Backup file from where to restore')
	bkr.add_argument('--non-interactive', action='store_true', help='Do not request confirmation')


	# ----------------------------------------
	# Create second level parsers for 'reorganize'
	# ----------------------------------------

	subparser = parser_reorg.add_subparsers(dest='subcommand')

	rgi = subparser.add_parser('images',  help="Reorganize images into observation nights")
	rgi.add_argument('-i', '--input-dir',  type=str, required=True , help='Images input directory')
	rgi.add_argument('-o','--output-dir', type=str, required=True , help='Images output base diretory')
	rgi.add_argument('-d', '--dry-run',   action='store_true', help='Show how many directories, do not copy.')


	# ----------------------------------------
	# Create second level parsers for 'session'
	# ----------------------------------------

	subparser = parser_session.add_subparsers(dest='subcommand')

	bcu = subparser.add_parser('current', help="session current list")
	bcu.add_argument('-x', '--extended',  action='store_true', help='Extended info')
	bcu.add_argument('--page-size',       type=int, default=10,  help="display page size")
   
	bli = subparser.add_parser('list', help="Batch list")
	bliex = bli.add_mutually_exclusive_group(required=True)
	bliex.add_argument('-b', '--session',  type=str , help='session identifier')
	bliex.add_argument('-a', '--all',  action='store_true' , help='all sessiones')
	bli.add_argument('-x', '--extended',  action='store_true', help='Extended info')
	bli.add_argument('--page-size',       type=int, default=10,  help="display page size")
   


	# -----------------------------------------
	# Create second level parsers for 'config'
	# -----------------------------------------

	subparser = parser_config.add_subparsers(dest='subcommand')

	cgl = subparser.add_parser('global',  help="Global configuration file actions")
	cglex = cgl.add_mutually_exclusive_group(required=True)
	cglex.add_argument('-c' ,'--create', action="store_true", help="Create global configuration file in user's HOME directory")
	cglex.add_argument('-l' ,'--list',   action="store_true", help="List current global configuration file template")
	cglex.add_argument('-d' ,'--diff',   action="store_true", help="Diff input config file with config file template")
	cgl.add_argument('-i', '--input-file', type=str , help='Input config file for diff')

	cca = subparser.add_parser('camera',  help="Create camera configuration file in user's HOME directory")
	ccaex = cca.add_mutually_exclusive_group(required=True)
	ccaex.add_argument('-c' ,'--create', action="store_true", help="Create camera configuration file in user's HOME directory")
	ccaex.add_argument('-l' ,'--list',   action="store_true", help="List current camera configuration file template")


	# ---------------------------------------
	# Create second level parsers for 'image'
	# ---------------------------------------
  
	subparser = parser_image.add_subparsers(dest='subcommand')
	parser_image.add_argument('--roi', type=mkrect1, metavar="<width,height>", help='Optional region of interest')


	ime = subparser.add_parser('list',    help='display image data')
	ime.add_argument('-a' ,'--all',       action="store_true", help="apply to all images in database")
	imeex = ime.add_mutually_exclusive_group(required=True)
	imeex.add_argument('--exif',      action="store_true", help="display EXIF metadata")
	imeex.add_argument('--generic',   action="store_true", help="display global metadata")
	imeex.add_argument('--state',     action="store_true", help="display processing state")
	imeex.add_argument('--data',      action="store_true", help="dark substracted signal averaged over roi")
	imeex.add_argument('--raw-data',  action="store_true", help="raw signal averaged over roi")
	imeex.add_argument('--dark',      action="store_true", help="raw signal of DARK images")
	imeex.add_argument('--dark-data', action="store_true", help="dark signal of LIGHT averaged over dark row or master dark")
	imeex.add_argument('--master',    action="store_true", help="display master dark data")
	ime.add_argument('--page-size',   type=int, default=10,  help="display page size")

	ird = subparser.add_parser('reduce',  help='run register/classify/stats</export pipeline')
	ird.add_argument('-w' ,'--work-dir',  type=str, required=True, help='Input working directory')
	ird.add_argument('-r' ,'--reset',     default=False, action="store_true",     help="Reprocess from start")
	ird.add_argument('-f' ,'--force-csv', default=False, action="store_true",     help="Force CSV file generration")
	ird.add_argument('-m' ,'--multiuser', default=False, action="store_true",     help="Multi-user reduction pipeline flag")
	irdex = ird.add_mutually_exclusive_group()
	irdex.add_argument('--csv-dir',        type=str, default=AZOTEA_CSV_DIR,  help='Optional directory where the CSV is placed')
	irdex.add_argument('--csv-subdir',     type=str, default="results",       help='(optional, multiuser), Leave CSV in a subdir for each observer')

	iex = subparser.add_parser('export',  help='export the whole database to a CSV file')
	iex.add_argument('--csv-file',        type=str, default=DEF_GLOBAL_CSV,  help='Optional session CSV file to export')
	return parser


# ================ #
# MAIN ENTRY POINT #
# ================ #

def main():
	'''
	Utility entry point
	'''
	try:
		options = createParser().parse_args(sys.argv[1:])
		setup(options)
		log.info("======= AZOTEA {0} =======".format(__version__))
		connection = open_database(DEF_DBASE)
		create_database(connection, SQL_SCHEMA, SQL_DATA_DIR, SQL_TEST_STRING)
		command      = options.command
		if command == 'init':
			return
		subcommand   = options.subcommand
		# Call the function dynamically
		func = command + '_' + subcommand
		globals()[func](connection, options)
	except KeyboardInterrupt as e:
		log.critical("[%s] Interrupted by user ", __name__)
	except Exception as e:
		log.critical("[%s] Fatal error => %s", __name__, str(e) )
		traceback.print_exc()
	finally:
		pass

main()



# -*- coding: utf-8 -*-
# ----------------------------------------------------------------------
# Copyright (c) 2020
#
# See the LICENSE file for details
# see the AUTHORS file for authors
# ----------------------------------------------------------------------

#--------------------
# System wide imports
# -------------------

import sys
import sqlite3
import os.path
import logging
import re
import glob

# Python3 catch
try:
    raw_input
except:
    raw_input = input 

# ---------------------
# Third party libraries
# ---------------------

# Access  template withing the package
from pkg_resources import resource_filename

import tabulate

#--------------
# local imports
# -------------


# ----------------
# Module constants
# ----------------

# -----------------------
# Module global variables
# -----------------------

log = logging.getLogger("azotea")

# ----------------------
# Module utility classes
# ----------------------

class Point(object):
    """ Point class represents and manipulates x,y coords. """
    def __init__(self, x=0, y=0):
        """ Create a new point at the origin """
        self.x = x
        self.y = y

    def __add__(self, rect):
        return NotImplemented

    def __repr__(self):
        return "({0},{1})".format(self.x, self.y)


class ROI(object):
    """ Region of interest  """

    PATTERN = r'\[(\d+):(\d+),(\d+):(\d+)\]'

    @classmethod
    def strproi(cls, roi_str):
        pattern = re.compile(ROI.PATTERN)
        matchobj = pattern.search(roi_str)
        if matchobj:
            x1 = int(matchobj.group(1))
            x2 = int(matchobj.group(2))
            y1 = int(matchobj.group(3))
            y2 = int(matchobj.group(4))
            return cls(x1,x2,y1,y2)
        else:
            return None


    def __init__(self, x1 ,x2, y1, y2):
        self.x1 = min(x1,x2)
        self.y1 = min(y1,y2)
        self.x2 = max(x1,x2)
        self.y2 = max(y1,y2)

    def dimensions(self):
        '''returns width and height'''
        return abs(self.x2 - self.x1), abs(self.y2 - self.y1)

    def __add__(self, point):
        return ROI(self.x1 + point.x, self.x2 + point.x, self.y1 + point.y, self.y2 + point.y)

    def __radd__(self, point):
        return self.__add__(point)
        
    def __repr__(self):
        return "[{0}:{1},{2}:{3}]".format(self.x1, self.x2, self.y1, self.y2)


class LogCounter(object):
    """ Point class represents and manipulates x,y coords. """
    def __init__(self, N, level=logging.INFO):
        """ Create a new point at the origin """
        self.N   = N
        self.lvl = level
        self.i   = 0

    def tick(self, *args, **kargs):
        lvl = kargs.get('level', self.lvl)
        self.i += 1
        if (self.i % self.N) == 0:
            log.log(lvl, *args, self.i)

    def end(self, *args):
        log.log(self.lvl, *args, self.i)
       


# -----------------------
# Module global functions
# -----------------------

def open_database(dbase_path):
    if not os.path.exists(dbase_path):
        with open(dbase_path, 'w') as f:
            pass
        log.info("Created database file {0}".format(dbase_path))
    return sqlite3.connect(dbase_path)


def create_database(connection, schema_path, data_dir_path, query):
    created = True
    cursor = connection.cursor()
    try:
        cursor.execute(query)
    except Exception:
        created = False
    if not created:
        with open(schema_path) as f: 
            lines = f.readlines() 
        script = ''.join(lines)
        connection.executescript(script)
        log.info("Created data model from {0}".format(os.path.basename(schema_path)))
        file_list = glob.glob(os.path.join(data_dir_path, '*.sql'))
        for sql_file in file_list:
            log.info("Populating data model from {0}".format(os.path.basename(sql_file)))
            with open(sql_file) as f: 
                lines = f.readlines() 
            script = ''.join(lines)
            connection.executescript(script)
        connection.commit()



def merge_two_dicts(d1, d2):
    '''Valid for Python 2 & Python 3'''
    merged = d1.copy()   # start with d1 keys and values
    merged.update(d2)    # modifies merged with d2 keys and values & returns None
    return merged


def chop(string, sep=None):
    '''Chop a list of strings, separated by sep and 
    strips individual string items from leading and trailing blanks'''
    chopped = [ elem.strip() for elem in string.split(sep) ]
    if len(chopped) == 1 and chopped[0] == '':
        chopped = []
    return chopped


if sys.version_info[0] < 3:
    # Python 2 version
    def packet_generator(iterable, size):
        '''Generates a sequence of 'size' items from an iterable'''
        finished = False
        while not finished:
            acc = []
            for i in range(0,size):
                try:
                    obj = iterable.next()
                except AttributeError:
                    iterable = iter(iterable)
                    obj = iterable.next()
                    acc.append(obj)
                except StopIteration:
                    finished = True
                    break
                else:
                    acc.append(obj)
            if len(acc):
                yield acc
else:
    # Python 3 version
    def packet_generator(iterable, size):
        '''Generates a sequence of 'size' items from an iterable'''
        finished = False
        while not finished:
            acc = []
            for i in range(0,size):
                try:
                    obj = iterable.__next__()
                except AttributeError:
                    iterable = iter(iterable)
                    obj = iterable.__next__()
                    acc.append(obj)
                except StopIteration:
                    finished = True
                    break
                else:
                    acc.append(obj)
            if len(acc):
                yield acc



def paging(iterable, headers, maxsize=10, page_size=10):
    '''
    Pages query output and displays in tabular format
    '''
    for rows in packet_generator(iterable, page_size):
        print(tabulate.tabulate(rows, headers=headers, tablefmt='grid'))
        maxsize -= page_size
        if len(rows) == page_size and maxsize > 0:
            raw_input("Press <Enter> to continue or [Ctrl-C] to abort ...")
        else:
            break



def mpl_style():
    mpl.rcParams['text.latex.unicode'] = True
    mpl.rcParams['text.usetex']        = False
    plt.rcParams['font.family']        = 'sans-serif'
    plt.rcParams['font.sans-serif']    = ['Verdana']
    plt.rcParams['font.size']          = 8  
    plt.rcParams['lines.linewidth']    = 4.
    plt.rcParams['axes.labelsize']     = 'small'
    plt.rcParams['grid.linewidth']     = 1.0
    plt.rcParams['grid.linestyle']     = ':'
    plt.rcParams['xtick.minor.size']   = 4
    plt.rcParams['xtick.major.size']   = 8
    plt.rcParams['ytick.minor.size']   = 4
    plt.rcParams['ytick.major.size']   = 8
    plt.rcParams['figure.figsize']     = 18,9
    plt.rcParams['figure.subplot.bottom'] = 0.15
    plt.rcParams['ytick.labelsize']     = 10
    plt.rcParams['xtick.labelsize']     = 10
    mpl.rcParams['xtick.direction']     = 'out'



# ----------------------------------------------------------------------
# Copyright (c) 2014 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------


#--------------------
# System wide imports
# -------------------

import os.path
import argparse
import errno


import logging

try:
    # Python 2
    import ConfigParser
except:
    import configparser as ConfigParser


#--------------
# local imports
# -------------

from .utils import chop, merge_two_dicts, ROI

# ----------------
# Module constants
# ----------------
# -----------------------
# Module global variables
# -----------------------

log = logging.getLogger("azotea")

# ------------------------
# Module Utility Functions
# ------------------------

def valueOrNone(string, typ):
    return None if not len(string) else typ(string)

def valueOrDefault(string, typ, default):
    return default if not len(string) else typ(string)

def load_config_file(filepath):
    '''
    Load options from configuration file whose path is given
    Returns a dictionary
    '''
    if filepath is None or not (os.path.exists(filepath)):
        raise IOError(errno.ENOENT,"No such file or directory", filepath)

    parser  = ConfigParser.RawConfigParser()
    # str is for case sensitive options
    parser.optionxform = str
    parser.read(filepath)
    log.info("Opening configuration file %s", filepath)

    options = {}
    options['observer']      = parser.get("observer","observer")
    options['organization']  = parser.get("observer","organization")
    options['email']         = parser.get("observer","email")
    options['focal_length']  = parser.get("camera","focal_length")
    options['f_number']      = parser.get("camera","f_number")
    options['bias']          = parser.get("camera","bias")
    options['location']      = parser.get("location","location")
    
    x0 = parser.get("image","x0"); x0 = valueOrDefault(x0, int, 0)
    y0 = parser.get("image","y0"); y0 = valueOrDefault(y0, int, 0)
    width  = parser.getint("image","width")
    height = parser.getint("image","height")
    options['roi']           = ROI(x0, x0 + width, y0, y0 + height)

    options['scale']         = parser.get("image","scale")
    options['dark_roi']      = parser.get("image","dark_roi")
    options['filter']        = parser.get("file","filter")

    # Handle empty keyword cases and transform them to None's
    options['bias']          = valueOrNone(options['bias'], int)
    options['focal_length']  = valueOrNone(options['focal_length'], int)
    options['f_number']      = valueOrNone(options['f_number'], float)
    options['scale']         = valueOrNone(options['scale'], float)
    options['email']         = valueOrNone(options['email'], str)
    options['organization']  = valueOrNone(options['organization'], str)
    options['dark_roi']      = valueOrNone(options['dark_roi'], str)

    return options


def merge_options(cmdline_opts, file_opts):
    # Read the command line arguments and config file options
    cmdline_opts = vars(cmdline_opts) # command line options as dictionary
    cmdline_set  = set(cmdline_opts)
    fileopt_set  = set(file_opts)
    conflict_keys = fileopt_set.intersection(cmdline_set)
    options      = merge_two_dicts(file_opts, cmdline_opts)
    # Solve conflicts due to the fact that command line always sets 'None'
    # for missing aruments and take precedence over file opts
    # in the above dictionary merge
    for key in conflict_keys:
        if cmdline_opts[key] is None and file_opts[key] is not None:
            options[key] = file_opts[key]
    options  = argparse.Namespace(**options)
    return options
